'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.JavaScriptFileRefactorer = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }(); // Dependencies:


var _esquery = require('esquery');

var _esquery2 = _interopRequireDefault(_esquery);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var JavaScriptFileRefactorer = exports.JavaScriptFileRefactorer = {
    identifierChange: identifierChange,
    metadataChange: metadataChange,
    referencePathChange: referencePathChange
};

function identifierChange(file, data) {
    var oldName = data.oldName,
        newName = data.newName,
        context = data.context;


    var query = 'Identifier[name="' + oldName + '"]';

    if (context) {
        query = context + ' > ' + query;
    }

    (0, _esquery2.default)(file.ast, query).forEach(function (identifier) {
        return identifier.name = newName;
    });
}

function metadataChange(file, data) {
    var oldName = data.oldName,
        newName = data.newName,
        type = data.type;
    var comments = file.ast.comments;

    if (!comments) {
        return;
    }

    var _comments = _slicedToArray(comments, 1),
        comment = _comments[0];

    if (!comment) {
        return;
    }

    var metaData = JSON.parse(comment.value);
    var item = metaData;
    if (type) {
        item = item[type].find(function (item) {
            return item.name === oldName;
        });
    }
    item.name = newName;
    comment.value = JSON.stringify(metaData);
}

function referencePathChange(file, data) {
    var oldFromPath = data.oldFromPath,
        newFromPath = data.newFromPath;

    if (!(oldFromPath && newFromPath)) {
        oldFromPath = newFromPath = data.fromPath;
    }

    var oldToPath = data.oldToPath,
        newToPath = data.newToPath;

    if (!(oldToPath && newToPath)) {
        oldToPath = newToPath = data.toPath;
    }

    var oldRequirePath = getRelativeRequirePath(_path2.default.dirname(oldFromPath), oldToPath);
    var newRequirePath = getRelativeRequirePath(_path2.default.dirname(newFromPath), newToPath);
    updatePaths(file, oldRequirePath, newRequirePath);
}

function getRelativeRequirePath(from, to) {
    var relativePath = _path2.default.relative(from, to).replace(/\\/g, '/');
    return (/^\./.test(relativePath) ? relativePath : './' + relativePath
    );
}

function updatePaths(file, oldRequirePath, newRequirePath) {
    var query = 'CallExpression[callee.name="require"] Literal[value="' + oldRequirePath + '"]';
    (0, _esquery2.default)(file.ast, query).forEach(function (requirePathLiteral) {
        requirePathLiteral.value = newRequirePath;
        requirePathLiteral.raw = '\'' + newRequirePath + '\'';
    });
}